@@ -1,5 +1,5 @@
 # Configuration file for pyB12MPS
-# Bridge12 Technologies, Inc. 2020
+# Bridge12 Technologies, Inc.
 
 # TCP Server
 HOST = 'localhost'
@@ -9,3 +9,7 @@ PORT = 50007
 defaultSerialPort = 'COM5'
 autoDetectSerialPort = True
 initializeOnStart = True
+serialDelay = 0.00 # in seconds
+
+# System Ready String
+systemReadyString = 'System Ready'
